package cz.mendelu.dnaAnalyser.utils.analyse.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import cz.mendelu.dnaAnalyser.sequence.Sequence;
import cz.mendelu.dnaAnalyser.user.User;
import cz.mendelu.dnaAnalyser.utils.model.Identifiable;
import cz.mendelu.dnaAnalyser.utils.model.Owned;
import cz.mendelu.dnaAnalyser.utils.model.Tagged;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.hibernate.annotations.CreationTimestamp;
import org.hibernate.annotations.Type;

import javax.persistence.*;
import java.util.Date;
import java.util.Set;
import java.util.UUID;


@Data
@Entity
@Inheritance(strategy = InheritanceType.JOINED)
@NoArgsConstructor(access = AccessLevel.PROTECTED)
@AllArgsConstructor(access = AccessLevel.PROTECTED)
@Table(name = "analyses")
public abstract class Analyse implements Identifiable, Owned, Tagged {

    @Id
    protected UUID id = UUID.randomUUID();

    @Temporal( TemporalType.TIMESTAMP )
    @CreationTimestamp
    protected Date created;

    @ManyToOne(cascade = CascadeType.DETACH)
    @JsonIgnore
    protected Sequence sequence;

    @ManyToOne(cascade = CascadeType.DETACH)
    @JsonIgnore
    protected User owner;

    @ElementCollection
    protected Set<String> tags;

    @Temporal( TemporalType.TIMESTAMP )
    protected Date finished;

    protected String title;

    protected Long resultCount;

    public UUID getSequenceId() {
        return sequence.getId();
    }

    public void finishNow() {
        this.finished = new Date();
    }


}
