package cz.mendelu.dnaAnalyser.sequence

import cz.mendelu.dnaAnalyser.sequence.data.SequenceDataRepository
import cz.mendelu.dnaAnalyser.user.User
import cz.mendelu.dnaAnalyser.user.UserRepository
import cz.mendelu.dnaAnalyser.utils.exception.NotFoundException
import spock.lang.Ignore
import spock.lang.Specification

class ModelSequenceServiceTest extends Specification {

    ModelSequenceService modelSequenceService

    SequenceRepository sequenceRepository
    SequenceDataRepository sequenceBufferRepository
    UserRepository userRepository


    UUID uuid = UUID.randomUUID()

    def setup() {
        modelSequenceService = new ModelSequenceService()
        modelSequenceService.sequenceRepository = sequenceRepository = Mock(SequenceRepository)
        //modelSequenceService.sequenceBufferRepository = sequenceBufferRepository = Mock(SequenceBufferRepository)
        //modelSequenceService.userRepository = userRepository = Mock(UserRepository)
    }

    def "test getRepository"() {
        when:
        def repository = modelSequenceService.getRepository()

        then:
        repository == sequenceRepository
    }

    def "test findAll"() {
        when:
        modelSequenceService.findAll(null, Collections.emptySet())

        then:
        1 * sequenceRepository.findAll(null)
    }

    def "test delete sequence with one user"() {
        given:
        def user = new User()
        def sequence = new Sequence(
                id: uuid,
                owner: user
        )

        when:
        modelSequenceService.delete(uuid)

        then:
        1 * sequenceRepository.findById(uuid) >> Optional.of(sequence)
        1 * sequenceRepository.save(sequence)
        //1 * sequenceBufferRepository.delete(sequence.getId())
    }

    @Ignore
    def "test delete sequence with more users"() {
        given:
        def user = new User(id: UUID.randomUUID())
        def sequence = new Sequence(
                id: uuid)
                .withAddOwner(user)
                .withAddOwner(new User(id: UUID.randomUUID()))

        when:
        modelSequenceService.delete(uuid)

        then:
        1 * sequenceRepository.findById(uuid) >> Optional.of(sequence)
        1 * sequenceRepository.save(sequence)
    }

    def "test findSequenceByUser"() {
        given:
        def sequence = new Sequence(
                id: UUID.randomUUID()
        )

        when:
        modelSequenceService.findOne(uuid)

        then:
        1 * sequenceRepository.findById(uuid) >> Optional.of(sequence)
    }

    def "test findSequenceByUser not found"() {
        when:
        modelSequenceService.findOne(uuid)

        then:
        1 * sequenceRepository.findById(uuid) >> Optional.empty()
        thrown(NotFoundException)
    }


}