package cz.mendelu.dnaAnalyser.utils.api;

import cz.mendelu.dnaAnalyser.jwt.JwtController;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.http.*;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Map;
import java.util.Objects;

@Component
public class ApiRestTemplate {

    @Value("${security.jwt.header}")
    private String tokenHeader;

    @Autowired
    private TestRestTemplate restTemplate;

    private String login;
    private String token;

    public void authentication(String login, String password) {
        if (Objects.equals(this.login, login) && token != null) {
            // Rest template is already authenticated
            return;
        }

        // Get JWT Token
        JwtController.LoginRequest loginQuery = new JwtController.LoginRequest();
        loginQuery.setLogin(login);
        loginQuery.setPassword(password);
        HttpEntity<JwtController.LoginRequest> loqinRequest = new HttpEntity<>(loginQuery);
        ResponseEntity<String> loginResponse = restTemplate.exchange("/api/jwt", HttpMethod.PUT, loqinRequest, String.class);

        // Test Server response
        if (loginResponse.getStatusCodeValue() != 201) {
            throw new cz.mendelu.dnaAnalyser.utils.api.AuthenticationException("Login failed", loginResponse);
        }

        // Store result
        this.login = login;
        this.token = loginResponse.getBody();
    }

    public void testAuthentication(String identification) {
        if (Objects.equals(this.login, identification) && token != null) {
            // Rest template is already authenticated
            return;
        }

        ResponseEntity<String> loginResponse = restTemplate.exchange("/api/jwt", HttpMethod.POST, null, String.class);

        // Test Server response
        if (loginResponse.getStatusCodeValue() != 201) {
            throw new cz.mendelu.dnaAnalyser.utils.api.AuthenticationException("Create test user failed", loginResponse);
        }

        // Store result
        this.login = identification;
        this.token = loginResponse.getBody();
    }

    public boolean isAuthenticated() {
        return login != null && token != null;
    }

    public ResponseEntity<Map> get(String url) {
        return exchange(url, HttpMethod.GET, new HttpHeaders(), null);
    }

    public ResponseEntity<Map> get(String url, MediaType mediaType) {
        HttpHeaders headers = new HttpHeaders();
        headers.setAccept(Arrays.asList(mediaType));
        return exchange(url, HttpMethod.GET, headers, null);
    }

    public ResponseEntity<Map> post(String url, Object body) {
        return exchange(url, HttpMethod.POST, new HttpHeaders(), body);
    }


    public ResponseEntity<Map> put(String url, Object body) {
        return exchange(url, HttpMethod.PUT, new HttpHeaders(), body);
    }

    public ResponseEntity<Map> delete(String url) {
        return exchange(url, HttpMethod.DELETE, new HttpHeaders(), null);
    }

    public ResponseEntity<Map> patch(String url, Object body) {
        return exchange(url, HttpMethod.PATCH, new HttpHeaders(), body);
    }

    private ResponseEntity<Map> exchange(String url, HttpMethod method, HttpHeaders httpHeaders, Object body) {
        if (isAuthenticated()) {
            httpHeaders.add(tokenHeader, token);
        }

        HttpEntity<?> request = new HttpEntity<>(body, httpHeaders);
        return restTemplate.exchange(url, method, request, Map.class);
    }



}
